# Copyright 2008 Canonical Ltd.  All rights reserved.

"""Tests of lazr.restful navigation."""

__metaclass__ = type

import unittest

from zope.interface import Interface, implements
from zope.publisher.interfaces import NotFound
from zope.schema import Text, Object

from lazr.restful.interfaces import IEntry
from lazr.restful.publisher import WebServicePublicationMixin


class IChild(Interface):
    one = Text(title=u'One')
    two = Text(title=u'Two')


class IChildEntry(IChild, IEntry):
    pass


class IParent(Interface):
    three = Text(title=u'Three')
    child = Object(schema=IChild)


class IParentEntry(IParent, IEntry):
    pass


class Child:
    implements(IChildEntry)
    schema = IChild

    one = u'one'
    two = u'two'


class Parent:
    implements(IParentEntry)
    schema = IParent

    three = u'three'
    child = Child()

    @property
    def context(self):
        return self


class FakeRequest:
    """A fake request satisfying `traverseName()`."""

    def getTraversalStack(self):
        return ()


class NavigationPublication(WebServicePublicationMixin):
    pass


class NavigationTestCase(unittest.TestCase):

    def test_toplevel_navigation(self):
        # Test that publication can reach sub-entries.
        publication = NavigationPublication()
        obj = publication.traverseName(FakeRequest(), Parent(), 'child')
        self.assertEqual(obj.one, 'one')

    def test_toplevel_navigation_without_subentry(self):
        # Test that publication raises NotFound when subentry attribute
        # returns None.
        parent = Parent()
        parent.child = None
        publication = NavigationPublication()
        self.assertRaises(
            NotFound, publication.traverseName,
            FakeRequest(), parent, 'child')


def additional_tests():
    return unittest.TestLoader().loadTestsFromName(__name__)
